import torch
import numpy as np
from .utils import *
from random import random

from maieutics.models.albert import *
from maieutics.models.bm25_retreiver import *

class Socrates:


    def __init__(self, corpus = 'SEP', model = 'albert'):
        
        self.corpus = corpus    

    def ask(self, question, num_paragraphs = 15, verbose = False):
        question = question.lstrip().rstrip()
        links, all_texts = get_url_text(question, self.corpus)

        if not links:
            print('No relevant articles found')
            return None, None, None
        
        res_albert = []
        res_texts = []
        res_links = []
        
        for i in range(len(links)):
            link = links[i]
            text = all_texts[i]
            if link != None:

                bm_1, _, _ = get_similarity([question], text)
                bm_1 = np.array(bm_1)
                bm_1_idx = bm_1[bm_1[:, 1] > 1.3-random()][:num_paragraphs, 0]  # two most similar
                bm_1_idx = np.array(bm_1_idx, dtype=int)
                
                text_ = ''.join(text[i] for i in sorted(bm_1_idx))
                text_ = remove_leading_space(text_)

            if len(bm_1_idx) == 0:
                continue

            # Generate response
            answer = answer(question, text_)
            if answer and (not 'could not find an answer' in answer):
                
                if verbose:
                    print('======= source =======')
                    print(link)     

                    print('======= TOP 3 BM25 SCORES =======')
                    print(bm_1[:3])

                    print('======= text =======')
                    print(text_)

                    print('======= anwser =======')
                    print(answer)
                
                res_albert.append(answer)

                res_texts.append(text_)

                res_links.append(link)
        
        return res_albert, res_texts, res_links



import requests
import bs4
import re
from googlesearch import search


domains = {'SEP' : 'plato.stanford.edu',
           'WIKI' : 'en.wikipedia.org'}


def remove_leading_space(text):
    i = 0
    try:
        while(text[i] == ' '):
            i+=1
        return text[i:]
    except:
        return text

def fix_links(links):
    if links:
        result = [links[0]]
        for i in range(1,len(links)):
            test = [True for elem in result if elem in links[i]]
            if not test:
                result.append(links[i])
        return result
    else: return links


def get_url_text(text, corpus):

    site = 'site:https://' + domains[corpus] + ' '
    print('Searching for relevant content in the '+corpus+'...')
    urls = search(site + text, tld='com', lang='en-US', stop=15)

    link = None


    links = [url for url in urls if domains[corpus] in url]
    links = fix_links(links)


    all_lines = []
    # Get text
    for link in links:

        response = requests.get(link)

        lines = []
        if response is not None:
            html = bs4.BeautifulSoup(response.text, 'html.parser')
            title = html.select("title")[0].text
            paragraphs = html.select("p")
            for para in paragraphs:
                text = re.sub(r'\[\w+\]', '', para.text)
                text = re.sub(r'\s\s+', '', text)
                text = re.sub(r'\n', ' ', text)
                if len(text.split()) > 5:
                    lines.append(text)
            all_lines.append(lines)
    if all_lines: 
        return links, all_lines
    else:
        print('No relevant articles found')
        return None, None



from setuptools import setup

setup(name='maieutics',
    version='0.0.11',
    description='Maieutics',
    packages=['maieutics'],
    author = 'Alfredo Lozano',
    author_email = 'lozanoa94@gmail.com',
    install_requires=[
   'beautifulsoup4==4.8.2',
   'bs4==0.0.1',
   'google==2.0.3',
   'nltk==3.4.5',
   'rank-bm25==0.2',
   'torch==1.4.0',
   'transformers==2.7.0'
   ],
   zip_safe=False)